@@ -1,6 +1,6 @@
 # --
 # Copyright (C) 2001-2021 OTRS AG, https://otrs.com/
-# Copyright (C) 2021-2022 Znuny GmbH, https://znuny.org/
+# Copyright (C) 2021 Znuny GmbH, https://znuny.org/
 # --
 # This software comes with ABSOLUTELY NO WARRANTY. For details, see
 # the enclosed file COPYING for license information (GPL). If you
@@ -19,12 +19,13 @@ use Kernel::System::VariableCheck qw(:all);
 
 our @ObjectDependencies = (
     'Kernel::Config',
+    'Kernel::Output::HTML::Layout',
     'Kernel::System::AutoResponse',
     'Kernel::System::CommunicationChannel',
     'Kernel::System::CustomerUser',
+    'Kernel::System::DateTime',
     'Kernel::System::DynamicField',
     'Kernel::System::DynamicField::Backend',
-    'Kernel::System::Encode',
     'Kernel::System::HTMLUtils',
     'Kernel::System::Log',
     'Kernel::System::Queue',
@@ -35,8 +36,6 @@ our @ObjectDependencies = (
     'Kernel::System::Ticket',
     'Kernel::System::Ticket::Article',
     'Kernel::System::User',
-    'Kernel::Output::HTML::Layout',
-    'Kernel::System::DateTime',
 );
 
 =head1 NAME
@@ -273,8 +272,8 @@ sub Signature {
 generate sender address (FROM string) for emails
 
     my $Sender = $TemplateGeneratorObject->Sender(
-        QueueID    => 123,
-        UserID     => 123,
+        QueueID => 123,
+        UserID  => 123,
     );
 
 returns:
@@ -359,7 +358,7 @@ sub Sender {
 generate template
 
     my $Template = $TemplateGeneratorObject->Template(
-        TemplateID => 123
+        TemplateID => 123,
         TicketID   => 123,                  # Optional
         Data       => $ArticleHashRef,      # Optional
         UserID     => 123,
@@ -367,7 +366,7 @@ generate template
 
 Returns:
 
-    $Template =>  'Some text';
+    $Template => 'Some text';
 
 =cut
 
@@ -385,10 +384,16 @@ sub Template {
         }
     }
 
-    my %Template = $Kernel::OM->Get('Kernel::System::StandardTemplate')->StandardTemplateGet(
+    my $StandardTemplateObject = $Kernel::OM->Get('Kernel::System::StandardTemplate');
+
+    my %Template = $StandardTemplateObject->StandardTemplateGet(
         ID => $Param{TemplateID},
     );
 
+    # Convert comma separated TemplateType to array for better searching.
+    my @TemplateTypes = split( /\s*,\s*/, $Template{TemplateType} );
+    $Template{TemplateType} = \@TemplateTypes;
+
     # do text/plain to text/html convert
     if (
         $Self->{RichText}
@@ -434,8 +439,9 @@ sub Template {
     }
 
     # If template type is 'Create' and there is customer user information, treat it as a ticket param in order to
-    # correctly replace customer user tags. See bug#14455.
-    if ( $Template{TemplateType} eq 'Create' && $Param{CustomerUserID} ) {
+    # correctly replace customer user tags.
+    my $IsCreateTemplate = grep { $_ eq 'Create' } @{ $Template{TemplateType} };
+    if ( $IsCreateTemplate && $Param{CustomerUserID} ) {
         $Ticket{CustomerUserID} = $Param{CustomerUserID};
     }
 
@@ -450,19 +456,20 @@ sub Template {
         Forward => 1,
         Note    => 1,
     );
+    my $IsSupportedType = grep { $SupportedTypes{$_} } @{ $Template{TemplateType} };
 
     my $TemplateText = $Template{Template} || '';
-    my $TemplateType = $Template{TemplateType};
+    my @TemplateType = @{ $Template{TemplateType} };
 
     # Remove unsupported tags only for some template types.
-    if ( !$SupportedTypes{ $Template{TemplateType} } ) {
+    if ( !$IsSupportedType ) {
         $TemplateText = $Self->_RemoveUnSupportedTag(
             Text                 => $Template{Template} || '',
             ListOfUnSupportedTag => \@ListOfUnSupportedTag,
         );
 
         # Reset template type for unsupported tag.
-        $TemplateType = '';
+        @TemplateType = ();
     }
 
     # replace place holder stuff
@@ -473,7 +480,7 @@ sub Template {
         Data       => $Param{Data} || {},
         UserID     => $Param{UserID},
         Language   => $Language,
-        Template   => $TemplateType,
+        Template   => \@TemplateType,
     );
 
     if ( $Self->{RichText} ) {
@@ -481,7 +488,6 @@ sub Template {
         $TemplateText =~ s/&gt;/>/g;
         $TemplateText =~ s/&quot;/"/g;
         $TemplateText =~ s/&apos;/'/g;
-        $TemplateText =~ s/&nbsp;/ /g;
     }
 
     return $TemplateText;
@@ -492,10 +498,10 @@ sub Template {
 generate internal or external notes
 
     my $GenericAgentArticle = $TemplateGeneratorObject->GenericAgentArticle(
-        Notification    => $NotificationDataHashRef,
-        TicketID        => 123,
-        UserID          => 123,
-        Data            => $ArticleHashRef,             # Optional
+        Notification => $NotificationDataHashRef,
+        TicketID     => 123,
+        UserID       => 123,
+        Data         => $ArticleHashRef,             # Optional
     );
 
 =cut
@@ -593,7 +599,7 @@ generate attributes
     my %Attributes = $TemplateGeneratorObject->Attributes(
         TicketID   => 123,
         ArticleID  => 123,
-        ResponseID => 123
+        ResponseID => 123,
         UserID     => 123,
         Action     => 'Forward', # Possible values are Reply and Forward, Reply is default.
     );
@@ -1676,6 +1682,17 @@ sub _Replace {
     # replace it
     $HashGlobalReplace->( $Tag, %Ticket, %DynamicFieldDisplayValues );
 
+    # OTRS_TICKET_LAST_ARTICLE_ID
+    my $ArticleObject = $Kernel::OM->Get('Kernel::System::Ticket::Article');
+    if ( $Ticket{TicketID} ) {
+        my @LastTicketArticle = $ArticleObject->ArticleList(
+            TicketID => $Ticket{TicketID},
+            OnlyLast => 1,
+        );
+        my $LastTicketArticleID = @LastTicketArticle ? $LastTicketArticle[0]->{ArticleID} : '';
+        $Param{Text} =~ s{$Start OTRS_TICKET_LAST_ARTICLE_ID $End}{$LastTicketArticleID}gixms;
+    }
+
     # COMPAT
     $Param{Text} =~ s/$Start OTRS_TICKET_ID $End/$Ticket{TicketID}/gixms;
     $Param{Text} =~ s/$Start OTRS_TICKET_NUMBER $End/$Ticket{TicketNumber}/gixms;
@@ -1704,9 +1721,12 @@ sub _Replace {
     # - if ArticleID is sent, data is from selected article.
     # - if ArticleID is not sent, data is from last customer/agent/any article.
     if ( $Param{Template} && $Ticket{TicketID} ) {
-        my $ArticleObject = $Kernel::OM->Get('Kernel::System::Ticket::Article');
 
-        if ( $Param{Template} eq 'Note' ) {
+        my $IsNoteTemplate    = grep { $_ eq 'Note' } @{ $Param{Template} };
+        my $IsAnswerTemplate  = grep { $_ eq 'Answer' } @{ $Param{Template} };
+        my $IsForwardTemplate = grep { $_ eq 'Forward' } @{ $Param{Template} };
+
+        if ($IsNoteTemplate) {
 
             # Get last article from agent.
             my @AgentArticles = $ArticleObject->ArticleList(
@@ -1740,7 +1760,7 @@ sub _Replace {
                 $Param{Data}->{Body}    = $CustomerArticle{Body};
             }
         }
-        elsif ( $Param{Template} eq 'Answer' || $Param{Template} eq 'Forward' ) {
+        elsif ( $IsAnswerTemplate || $IsForwardTemplate ) {
 
             # If $Param{Data} has agent article data, we will set subject and body in $Param{DataAgent}
             # to values from $Param{Data} in order to right replacing of OTRS_AGENT_SUBJECT/BODY tags.
